/*
    This file is part of JLog.

    Created by Glendon Holst for Alan Mackworth and the 
    "Computational Intelligence: A Logical Approach" text.
    
    Copyright 1998, 2000, 2002 by University of British Columbia and 
    Alan Mackworth.
    
    This notice must remain in all files which belong to, or are derived 
    from JLog.
    
    Check <http://jlogic.sourceforge.net/> or 
    <http://sourceforge.net/projects/jlogic> for further information
    about JLog, or to contact the authors.
     
    JLog is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    JLog is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with JLog; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    URLs: <http://www.fsf.org> or <http://www.gnu.org>
*/
//#########################################################################
//	DBExecuteGoal
//#########################################################################
 
package ubc.cs.JLog.Extras.DataBase;

import java.sql.*;
import java.util.*;
import ubc.cs.JLog.Terms.*;
import ubc.cs.JLog.Foundation.*;
import ubc.cs.JLog.Builtins.*;

public class jDBExecuteGoal extends jGoal 
{
 protected jTerm		sql = null;

 public jDBExecuteGoal(jTerm t) 
 {
  sql = t;
 };
 
 public boolean prove(iGoalStack goals, iGoalStack proved) 
 {String	query = null;
  jTerm		sql_term = sql.getTerm();

  if (sql_term.type == iType.TYPE_VARIABLE) 
   throw new RuntimeException("Query is unbound");
  
  query = sql_term.toString();
  
  try 
  {Statement		stmt = jDBConnect.getConnection().createStatement();
   int				rows = stmt.executeUpdate(query);

   proved.push(this);
   return true;
  } 
  catch (SQLException sqlex) 
  {
   goals.push(this);
   return false;
  }
 };

 public boolean 	retry(iGoalStack goals,iGoalStack proved)
 {
  goals.push(this); // a retry that follows may need a node to remove or retry
  return false;
 };

 public String 		getName() 
 {
  return "db_execute";
 };
 
 public int 		getArity() 
 {
  return 1;
 };
 
 public String 		toString() 
 {StringBuffer 	sb = new StringBuffer();
   
  sb.append(getName()+"/"+String.valueOf(getArity()));
  if (sql != null)
   sb.append(" goal: "+getName()+"("+sql.toString()+")");
  
  return sb.toString();
 };
}

