/**
 * 
 */
package hipaa;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.io.Serializable;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;

import policy.HIPAAQuery;
import policy.HIPAAResult;

import ubc.cs.JLog.Foundation.iPrologFileServices;
import ubc.cs.JLog.Foundation.jPrologAPI;
import ubc.cs.JLog.Foundation.jPrologFileServices;

/**
 * @author Nipun Dave
 *
 */
public class GenerateHIPAAPolicy implements Serializable {

		/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public static String logFileName = "src/hipaa/log.txt";
	public static String prologPolicyFileName = "Policy/HIPAAPolicy.pl";
	public static String policyStoreFileName = "Policy/HIPAAPolicyJava.ser";
	public static boolean logWrite = false;

	public static void main(String[] args) throws IOException {
		String[] roles = {"chief_of_medicine", "clergy", "cop", "coveredEntity", "dad", "dead", "doctor", "government_official",
				"government_agency_health_plan", "group_health_plan", "healthCare_plan", "healthCare_clearing_house", 
				"health_insurance_issuer", "health_oversight_agency", "healthCare_provider", "healthCare_accreditation_organization", 
				"hospital", "intern", "kid", "legal_attorney", "mom", "patient", "public_health_authority", 
				"secretary", "surgeon", "teen", "thief", "trainee", "volunteer"};
		
		String[] owners = {"coveredEntity", "clergy", "dad", "dead", "chief_of_medicine", "doctor", 
				"intern", "kid", "mom", "patient", "surgeon", "teen", "thief" , "trainee", "volunteer"};
		
		//String[] types = {"phi"};
		
		String[] purposes = {"compliance", "create_deidentified_info", "create_protected_health_info",  
				"definition_healthCare_operations", "determining_legal_options", "healthCare_fraud_abuse_detection", 
				"healthCare_operations", "incident_to_use", "investigate", "payment", 
				"receive_deidentified_info", "requested_by_Individual", "standards_failure_misconduct","treatment"};
		
		//String[] consents = {"consented", "opp_given"};
		
		iPrologFileServices fs 	= new jPrologFileServices();
		PrintWriter	out 		= new PrintWriter(System.out);
		BufferedReader in 		= new BufferedReader(new InputStreamReader(System.in));
		InputStream	is 			= new FileInputStream(new File(prologPolicyFileName));
		jPrologAPI api 			= new jPrologAPI(is,fs,out,in,null);
		Hashtable HTresult;
		
		HIPAAQuery q = new HIPAAQuery();
        HIPAAResult result = new HIPAAResult();
        HashMap<String, Set<HIPAAResult>> policy = new HashMap<String, Set<HIPAAResult>> ();
        
        Set<HIPAAResult> setOfResults = new HashSet<HIPAAResult> ();
		
		for (String sender : roles){
			for (String owner : owners) {
				for (String purpose : purposes){
					setOfResults = new HashSet<HIPAAResult> ();
					q = new HIPAAQuery(sender, owner, "phi", purpose);
					HTresult = api.query(q.prologQuery());
					
					while (HTresult != null)
					{
						result = new HIPAAResult(q, HTresult.get("R").toString(), HTresult.get("B").toString(), HTresult.get("C").toString());
						setOfResults.add(result);
						//if (logWrite) {log.write(query + " : " +result.toString() + "\n");}
						HTresult = api.retry();
					}
					
					policy.put(q.toString(), setOfResults);
				}
			}
		}
		System.out.println("Generated the policy.");
		serializeObject(policy, policyStoreFileName);
		System.out.println("Stored the policy.");
		//log.close();
	}
	
	public static void serializeObject(Object obj, String fileName){
		try {
			FileOutputStream fos = new FileOutputStream(fileName);
			ObjectOutputStream oos = new ObjectOutputStream(fos);
			oos.writeObject(obj);
			oos.close();
		} 
		catch (IOException e) {
			e.printStackTrace();
		}
	}

}
