package server.process;

import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Random;
import java.util.zip.CRC32;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;
import javax.servlet.http.HttpServletRequest;
import policy.HIPAAQuery;
import security.abemodule.keygeneration.KeyGenerator;
import settings.HIPAA_ABE_Settings;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;
import crypto.DecryptionModule;
import crypto.EncryptionModule;

public class ABERequest {
	private String jobName = "None";
	ProcessInput processInput;
	long startTime;
	long finishTime;
	long numjobs = 0;
	long completed = 0;
    protected static int numOfEncryptedVersionsOfKey = 0;  
    private static String randomFolderName = new String();
    public static HashMap<String, ABERequest> abejobs = new HashMap<String, ABERequest>();
    
    public ABERequest(ProcessInput processInput2, String reqid) {
		startTime = System.currentTimeMillis();
	}

	public String getJobDescription()
    {
    	return "Description: " + jobName + " status= " + ( completed == 100 ? "Done, time= " + finishTime + " ms <" + numjobs + "," + finishTime + ">": ( "Running " + completed + "% done, elapsed time=" 
               + ((System.currentTimeMillis() - startTime )/1000) + " s"));
    }

	public void processEncrypt(HttpServletRequest request, String workingPath,
			PrintWriter out) throws Exception {
		String sender = request.getParameter("sender");
		String owner = request.getParameter("owner");
		String purpose = request.getParameter("purpose");
		String message = request.getParameter("message");
		String reqid = request.getParameter("reqid");
		
		this.jobName = "Encryption: sender= " + sender + " owner= " + owner;
		abejobs.put(reqid, this);
		System.out.println("My job name is " + getJobDescription());
		workingPath += "-enc";
		(new File(workingPath)).mkdirs();
		
		String UUmessage = encryptMessage(sender, owner, purpose, message, workingPath, this);
         
		out.println("abeMessage|" + UUmessage.replaceAll("/", "%2F")+"|statusText|Done " + System.currentTimeMillis());
		completed = 100;
		finishTime = (System.currentTimeMillis() - startTime);
	}

	public void processDecrypt(HttpServletRequest request, String workingPath,
			PrintWriter out) throws Exception {
		String uuMsg = request.getParameter("uuMsg");
		String abekey = request.getParameter("abekey");
		
		this.jobName = "Decryption: key= " + abekey.substring(0,10) + "...";
		
		workingPath += "-dec";
		(new File(workingPath)).mkdirs();

		if( uuMsg.length() > 0 ) {
			System.out.println("Accepted uuMsg-" + uuMsg.length() + ": " + uuMsg.substring(0,Math.min(uuMsg.length(),100)));
			System.out.println("ABE key: " + abekey);
			
			workingPath = processUUencodedArchive(uuMsg, abekey, workingPath);
		}
		String resp2 = decryptMessage(workingPath);
		out.println("decryptedMessage|" + resp2);
		System.out.println("Decrypted: " + resp2);
	}

	public void processKeygen(HttpServletRequest request, String workingPath,
			PrintWriter out) throws FileNotFoundException, IOException {
		String receiver = request.getParameter("receiver");
		String belief = request.getParameter("belief");
		
		this.jobName = "Keygen: receiver= " + receiver + " belief= " + belief;

		if (request.getParameter("belief").equals("minimum_necessary_to_purpose"))
			belief = request.getParameter("belief")+request.getParameter("receiver");
		String consent = request.getParameter("consent");
		
		String policies = "RX" + receiver + " and " + "CX" + consent + " and " + "BX" + belief;
		String attributes = "RX" + receiver + ",CX" + consent + ",BX" + belief;

		workingPath += "-gen";
		(new File(workingPath)).mkdirs();

		KeyGenerator keygenerator = new KeyGenerator(workingPath, HIPAA_ABE_Settings.locationofABETools);	
		keygenerator.gotokeygeneratorpath();
		keygenerator.generateABEKey("accessorKey.key", attributes,policies);
		FileReader keyfileStream = new FileReader( workingPath + "/accessorKey.key");
		String keyval = "";
		
		while(keyfileStream.ready()) {
			char buffer[] = new char[16];
			keyfileStream.read(buffer);
			keyval += new String(buffer);
		}
		
		System.out.println("Read key="+keyval);
		out.println("keyval|"+keyval);
	}

	protected String encryptMessage(String sender, String owner, String purpose, String message, String fileLoc, ABERequest abeRequest) throws Exception{
		EncryptionModule encMod = new EncryptionModule();
		randomFolderName = Integer.toString(Math.abs(new Random().nextInt(1048576)));
		
		String path = fileLoc + "/" + randomFolderName;
		new File(path).mkdir();
		
		encMod.encryptedKeyFileName = path + "/aesKey.key";
		encMod.encryptedKeyPath = path;
		encMod.encryptedFileName 		= path + "/encryptedMssg.txt";
		encMod.abeEncryptedKeyName		= path + "/abeEncryptedAESKey";
		encMod.q 						= new HIPAAQuery(sender, owner, "phi", purpose);
		System.out.println(encMod.q.toString());
		System.out.println(encMod.encryptedFileName);	
		numOfEncryptedVersionsOfKey = encMod.encrypt(message, abeRequest);
		System.out.println(message);
		
		String abefile = fileLoc + "/abe-" + randomFolderName + ".abe";
		(new File(path + "/aesKey.key")).delete();
		createZipArchive(fileLoc, abefile, path);
		String UUString = "";

		byte[] zipbytes;
		try {
			FileInputStream fs = new FileInputStream(abefile);
			
			zipbytes = new byte[fs.available()];
			fs.read(zipbytes);
			UUString = (new BASE64Encoder()).encode(zipbytes);
			System.out.println("Created ABE UUencoded text:" + UUString.substring(0, Math.min(UUString.length(),200)));
		} catch (Exception e) {
			e.printStackTrace();
		}

		CRC32 crc = new CRC32();
		try {
			crc.update(UUString.getBytes("UTF8"));
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		
		return HIPAA_ABE_Settings.abeMsgStart + "\n" + UUString +
				"\n" + HIPAA_ABE_Settings.abeMsgEnd + "\n" + HIPAA_ABE_Settings.abeCRC + crc.getValue() + "\n";
	}
	
	protected void createZipArchive(String path, String zipname, String fileLoc) {
		try {
			byte buffer[] = new byte[1024];
			
			System.out.println("Creating " + zipname + " from " + fileLoc);
			ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(zipname));
			File zipDir = new File(fileLoc);
			String zipFiles[] = zipDir.list();
			
			for(int i = 0; i < zipFiles.length; ++i) {
				System.out.println("Adding " + fileLoc + "/" + zipFiles[i]);
				
				ZipEntry ze = new ZipEntry(zipFiles[i]);
				zos.putNextEntry(ze);
				FileInputStream origin = new FileInputStream(fileLoc + "/" + zipFiles[i]);
				int count;
				while((count = origin.read(buffer, 0, buffer.length)) != -1) {
					zos.write(buffer, 0, count);
				}
			}
			
			zos.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	protected String processUUencodedArchive(String message, String abekey, String path) throws Exception {
		String outputFolder = "output-" + Integer.toString(Math.abs(new Random().nextInt(1048576)));
		String workingPath = path + "/" + outputFolder + "/";
 
		try {
			File dir = new File( workingPath );
			dir.mkdirs();
			
			FileWriter fs = new FileWriter( workingPath + "accessorKey.key" );
			fs.write( abekey );
			fs.close();
		} catch (IOException e) {
			System.err.println("Failed to UUdecode file");
			e.printStackTrace();
			throw new Exception();
		}
		
		String uuMessageParts[] = message.split("\n");
		if(!uuMessageParts[0].equals(HIPAA_ABE_Settings.abeMsgStart) ||
		    uuMessageParts[uuMessageParts.length - 2].equals(HIPAA_ABE_Settings.abeMsgStart))
		{
			System.err.println("Invalid UUE Message: " + message);
		}
		
		Long crcval = Long.parseLong(uuMessageParts[uuMessageParts.length - 1].replace(HIPAA_ABE_Settings.abeCRC,""));
		CRC32 crc = new CRC32();
		String uuMsg = "";
		int beginLine = Arrays.<String>asList(uuMessageParts).indexOf(HIPAA_ABE_Settings.abeMsgStart);
		int endLine = Arrays.<String>asList(uuMessageParts).indexOf(HIPAA_ABE_Settings.abeMsgEnd);
		
		if( beginLine == -1 || endLine == -1 ) {
			System.err.println("Malformed UUE string. Begin/end line not found.");
			abejobs.remove(this);
			throw new Exception();
		}
								
		for(int i = beginLine + 1; i <= endLine /* begin, end, crc */; ++i) {
			uuMsg += uuMessageParts[i] + "\n";
		}
		System.out.println("start\n" + uuMsg + "\nend");
		
		try {
			crc.update(uuMsg.getBytes("UTF8"));
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		
		if(crcval != crc.getValue()) {
			System.err.println("CRC value mismatch, stored CRC " + crcval + " != computed CRC " + crc.getValue());
		}
		
		BASE64Decoder decoder = new BASE64Decoder();
				
		try {
			byte uuMsgBytes[] = decoder.decodeBuffer(uuMsg);
			FileOutputStream f = new FileOutputStream(workingPath + "/sharps-tmp.zip");
			f.write(uuMsgBytes);
			f.close();
			
			ZipInputStream zi = new ZipInputStream(new ByteArrayInputStream(uuMsgBytes));
			
			ZipEntry ze;
			while((ze = zi.getNextEntry()) != null) {
				System.out.println("Found zip entry: " + ze.getName());
				
				byte[] buffer = new byte[16 * 1024];
				String fileNameParts[] = ze.getName().split("/");
				String fileName = workingPath + "/" + fileNameParts[fileNameParts.length -1];
				System.out.println("Writing to " + fileName);
				FileOutputStream fos = new FileOutputStream( fileName );
				int count;
				while((count = zi.read(buffer, 0, buffer.length)) != -1) {
					fos.write(buffer, 0, count);
				}
				fos.flush();
				fos.close();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		System.out.println("Working folder: " + workingPath);
		return workingPath;
	}
	
	protected String decryptMessage(String filePath){
		DecryptionModule decMod = new DecryptionModule();
		
		decMod.userKeyPath 				= filePath;
		decMod.userKeyName 				= "accessorKey.key";
		decMod.abeEncryptedKeyPath 		= filePath;
		decMod.abeEncryptedKeyName 		= "abeEncryptedAESKey";
		decMod.aesEncryptedFilePath 	= filePath;
		decMod.aesEncryptedFileName 	= "/encryptedMssg.txt";
		decMod.decryptedFilePath 		= filePath;
		decMod.decryptedFileName 		= "/decryptedMssg.txt";
		boolean found = decMod.decrypt();
		if (!found)
			return "Sorry, with your privileges, file cannot be accessed.";
		else{
			String result ="";
			try{
				byte[] buf = new byte[1024];
				FileInputStream in = new FileInputStream(decMod.decryptedFilePath + "/" + decMod.decryptedFileName);
				while ((in.read(buf)) >= 0) {
					result = result + new String (buf);
				}
				in.close();
			}
			catch (Exception e){
			}
			return result; 
		}
	}
	
	protected boolean deleteFile(String filePath){
		File file = new File(filePath);
		
		if (file.exists() && file.isFile()){
			return file.delete();
		}
		
		return false;
	}
	
	protected boolean storeStringInFile(String str, String filePath){
		try {
			FileWriter fstream = new FileWriter(filePath);
			BufferedWriter out = new BufferedWriter(fstream);
			out.write(str);
			out.close();
			return true;
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
	}
	
	protected String readFile(String filePath){
		StringBuffer content = new StringBuffer();
		FileInputStream fis = null;
		int ch;
		try {
			fis = new FileInputStream(new File(filePath));
			while((ch = fis.read()) != -1){
				content.append((char)ch);
			}
			fis.close();
		} catch (FileNotFoundException e) {
			System.out.println("File " + filePath + " was not found.");
			return null;
		} catch (IOException e) {
			return null;
		}
		return content.toString();
	}

	public String getStatusString() {
    	return getJobDescription();
	}

	public static ABERequest lookup(String reqid) {
		return abejobs.get(reqid);
	}

	public static String getStatusAllString() {
    	String status = "Status time=" + System.currentTimeMillis() + " jobs=" + abejobs.size() + "\n";
    	
    	for (Iterator<String> i = abejobs.keySet().iterator(); i.hasNext(); ) {
    		status += " * " + abejobs.get( i.next() ).getJobDescription() + "\n";
    	}
    	
		return status;
	}

	public void setCompletion(int i) {
		System.out.println("Set job completion to: " + i);
		completed = i;
	}
	
	public void setNumjobs(int i) {
		System.out.println("Set number of jobs to: " + i);
		numjobs = i;
	}

	public String getStatusVal() {
		return Long.toString(completed);
	}
}
